@@ -7,9 +7,8 @@ from napari_sim_processor.widget_settings import Setting, Combo_box
 from napari_sim_processor.baseSimProcessor import pytorch, cupy
 from napari_sim_processor.hexSimProcessor import HexSimProcessor
 from napari_sim_processor.convSimProcessor import ConvSimProcessor
-# from napari_sim_processor.simProcessor import SimProcessor
 import napari
-from qtpy.QtWidgets import QVBoxLayout,QSplitter, QHBoxLayout, QWidget, QPushButton, QLineEdit
+from qtpy.QtWidgets import QVBoxLayout, QSplitter, QHBoxLayout, QWidget, QPushButton, QLineEdit
 from qtpy.QtCore import Qt
 from napari.layers import Image
 import numpy as np
@@ -334,15 +333,30 @@ class SimAnalysis(QWidget):
            
             
     @qthrottled (timeout=10)
-    def on_step_change(self, *args):   
-        if hasattr(self, 'imageRaw_name'):
+    def on_step_change(self, *args):
+        if self.is_image_in_layers():
             t0 = time.time()
-            self.setReconstructor()
-            min_timeout = 10 #ms
+            if self.step_changed(0,2,3,4):
+                self.setReconstructor()
             delta_t = time.time() - t0
-            self.on_step_change.set_timeout(delta_t + min_timeout)
+            self.on_step_change.set_timeout(delta_t)
 
 
+    def step_changed(self, *step_indexes):
+        """
+        Returns True if any of the steps with indexes in step_indexes has changed
+        """
+        if not hasattr(self, '_step'):
+            self._step = (0,0,0,0,0)
+        current_step = self.viewer.dims.current_step
+        changed = False
+        for step_idx in step_indexes:
+            if self._step[step_idx] != current_step[step_idx]:
+                changed = True
+        self._step = current_step
+        return changed
+            
+   
     def show_image(self, image_values, im_name, **kwargs):
         '''
         creates a new Image layer with image_values as data
@@ -497,7 +511,7 @@ class SimAnalysis(QWidget):
         if hasattr(self, 'h'):
             delattr(self, 'h')
 
-    
+   
     def setReconstructor(self,*args):
         '''
         Sets the attributes of the Processor
@@ -581,14 +595,15 @@ class SimAnalysis(QWidget):
             elif not self.showXcorr.val and imname in self.viewer.layers:
                 self.remove_layer(self.viewer.layers[imname])
     
+    
     def show_functions(self, *args):
         self.show_wiener()
         self.show_spectrum()
         self.show_xcorr()
         self.show_carrier()
         self.show_eta()
-        
-     
+
+    
     def calculate_kr(self,N):  
         '''
         Parameter:
@@ -605,7 +620,7 @@ class SimAnalysis(QWidget):
         cutoff_in_pixels = cutoff / dk
         return cutoff_in_pixels, dk   
       
-     
+    
     def show_carrier(self, *args):
         '''
         Draws the carrier frenquencies in a shape layer
@@ -630,7 +645,7 @@ class SimAnalysis(QWidget):
             elif name in self.viewer.layers:
                 self.remove_layer(self.viewer.layers[name])
     
-                   
+             
     def show_eta(self):
         '''
         Shows two circles with radius eta (green circle), 
@@ -656,7 +671,7 @@ class SimAnalysis(QWidget):
             elif name in self.viewer.layers:
                 self.remove_layer(self.viewer.layers[name])   
 
-    
+
     def add_circles(self, locations, radii,
                     shape_name='shapename', color='blue', hold=False):
         '''
@@ -808,6 +823,7 @@ class SimAnalysis(QWidget):
         @thread_worker(connect={'returned': update_sim_image})
         def _stack_reconstruction():
             warnings.filterwarnings('ignore')
+            start_time = time.time()
             stackSIM = np.zeros([sz,2*sy,2*sx], dtype=np.single)
             for zidx in range(sz):
                 phases_stack = np.squeeze(pa_stack[:,zidx,:,:])
@@ -833,6 +849,8 @@ class SimAnalysis(QWidget):
                     stackSIM[zidx, :, :] = self.h.reconstruct_cupy(phases_stack.astype(np.float32))  # TODO:this is left after conversion from torch
                 else:
                     stackSIM[zidx,:,:] = self.h.reconstruct_rfftw(phases_stack)      
+            elapsed_time = time.time() - start_time
+            self.messageBox.setText(f'Stack reconstruction time {elapsed_time:.3f}s')
             return stackSIM
         
         @thread_worker(connect={'returned': update_sim_image})
@@ -840,7 +858,7 @@ class SimAnalysis(QWidget):
             warnings.filterwarnings('ignore')
             start_time = time.time()
             if self.proc.current_data == Accel.USE_TORCH.value:
-                stackSIM = self.h.batchreconstructcompact_pytorch(paz_stack, blocksize = 32)
+                stackSIM = self.h.batchreconstructcompact_pytorch(paz_stack, blocksize=32)
             elif self.proc.current_data == Accel.USE_CUPY.value:
                 stackSIM = self.h.batchreconstructcompact_cupy(paz_stack, blocksize=32)
             else:
@@ -848,7 +866,7 @@ class SimAnalysis(QWidget):
             elapsed_time = time.time() - start_time
             self.messageBox.setText(f'Batch reconstruction time {elapsed_time:.3f}s')
             return stackSIM
-        
+
         # main function executed here
         assert self.isCalibrated, 'SIM processor not calibrated, unable to perform SIM reconstruction'
         fullstack = self.get_hyperstack()
